#include <avr/pgmspace.h> // allows use of PROGMEM to store patterns in flash

#define CUBESIZE 4
#define PLANESIZE CUBESIZE*CUBESIZE
#define PLANETIME 3333 // time each plane is displayed in us -> 100 Hz refresh
#define TIMECONST 10 // multiplies DisplayTime to get ms - why not =100?


/*
** Defining pins in array makes it easier to rearrange how cube is wired
** Adjust numbers here until LEDs flash in order - L to R, T to B
** Note that analog inputs 0-5 are also digital outputs 14-19!
** Pin DigitalOut0 (serial RX) and AnalogIn5 are left open for future apps
*/

int LEDPin[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
int PlanePin[] = {16, 17, 18, 19};

#include "LedPatterns.h"

//----------------------------
// Initialisation du programme
//----------------------------
void setup() {
int pin; 
  // Initialise les 16 pins en sortie pour les colonnes
  // Actif=HIGH
  for (pin=0; pin<PLANESIZE; pin++) {
    pinMode( LEDPin[pin], OUTPUT );
  }
  // Initialise les plans
  // Actif=HIGH
  for (pin=0; pin<CUBESIZE; pin++) {
    pinMode( PlanePin[pin], OUTPUT );
  }
}

void Animation(int indexdeb, int indexfin, int nbrepeat) {
byte PatternBuf[PLANESIZE]; // buffer de travail
int PatternIdx;             // Index dans le buffer
int PatternIdxFin;             // Index dans le buffer

byte DisplayTime;           // temps affichage d'un pattern
unsigned long EndTime;
int plane;                  // Compteur pour rafraichissement du cube
int patbufidx;              // index dans le buffer
int ledrow;                 // Compteur de ligne
int ledcol;                 // Compteur de colonne
int ledpin;                 // Compte les leds

  for (int i=0; i< nbrepeat; i++) {
    // Initialise l'index dans la table des patterns
    PatternIdx = indexdeb*17;
    PatternIdxFin=indexfin*17; 
               

    // boucle d'affichage 
    // Arret si temps a 0 dans la table des patterns
    do {
      // Lit un pattern et recopie dans le buffer de travail
      memcpy_P( PatternBuf, PatternTable+PatternIdx, PLANESIZE );
      PatternIdx += PLANESIZE;
      // Lecture du temps d'affichage du pattern (dernier octet de chaque pattern)
      DisplayTime = pgm_read_byte_near( PatternTable + PatternIdx++ );
      // Calcul le temps final pour la fin d'affichage du pattern
      EndTime = millis() + ((unsigned long) DisplayTime) * TIMECONST;
  
      // loop while DisplayTime>0 and current time < EndTime
      while ( millis() < EndTime ) {
        patbufidx = 0; // reset index counter to beginning of buffer
        // loop over planes
        for (plane=0; plane<CUBESIZE; plane++) {
          // turn previous plane off
          if (plane==0) {
            digitalWrite( PlanePin[CUBESIZE-1], LOW );
          } 
          else {
            digitalWrite( PlanePin[plane-1], LOW );
          }

          // load current plane pattern data into ports
          ledpin = 0;
          for (ledrow=0; ledrow<CUBESIZE; ledrow++) {
            for (ledcol=0; ledcol<CUBESIZE; ledcol++) {
              digitalWrite( LEDPin[ledpin++], PatternBuf[patbufidx] & (1 << ledcol) );
            }
            patbufidx++;
          } 
          // active le plan
          digitalWrite( PlanePin[plane], HIGH );
          // Delay pour rester allumé
          delayMicroseconds( PLANETIME );
          
        }
      } 
    } while (PatternIdx <= PatternIdxFin); // Jusqu'au derrier pattern
    // desactive le dernier plan allume
     digitalWrite( PlanePin[plane-1], LOW );
    
  }
}

//---------------------------------------
// Affichage croix pharmacie
//---------------------------------------
void AffPharmacie(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(328,333,1);
  }
} 
//---------------------------------------
// Affichage de deux fleches alternatives
//---------------------------------------
void AffCarreCroix(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(326,327,1);
  }
} 
//---------------------------------------
// Affichage de deux fleches alternatives
//---------------------------------------
void AffFleches(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(311,325,1);
  }
}
//---------------------------------------
// Affichage de deux plans tournants
//---------------------------------------
void AffPlansTourne(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(305,310,1);
  }
}
//---------------------------------------
// Affichage dun cube tournant
//---------------------------------------
void AffCarreRespire(int nbfois) {
   Animation(291,300,1);
   for (int i=0; i<nbfois; i++) {
      Animation(295,300,2);
  }
   Animation(301,304,1);
    
}


//---------------------------------------
// Affichage dun cube tournant
//---------------------------------------
void AffPetitsCubesTourne(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(287,290,1);
  }
}


//---------------------------------------
// Affichage dun cube tournant
//---------------------------------------
void AffCubeTourne(int nbfois) {
   Animation(268,278,1);
   for (int i=0; i<nbfois; i++) {
      Animation(276,278,2);
  }
   Animation(279,286,1);
    
}

//---------------------------------------
// Affichage dun cube en ressort
//---------------------------------------
void AffCarreRessort(int nbfois) {
   Animation(245,267,1);
   for (int i=0; i<nbfois; i++) {
      Animation(256,267,1);
  }  
}
//---------------------------------------
// Affichage dun serpent en hauteur
//---------------------------------------
void AffSerpentHauteur(int nbfois) {
   Animation(210,236,1);
   for (int i=0; i<nbfois; i++) {
      Animation(213,236,1);
  }  
   Animation(237,244,1);
}
//---------------------------------------
// Affichage dun cube qui se remplit
// en tournant et se vide
//---------------------------------------
void AffGeyser(int nbfois) {
   Animation(200,209,1);
   for (int i=0; i<nbfois; i++) {
      Animation(202,209,1);
  }  
}
//---------------------------------------
// Affichage dun cube qui se remplit
// en tournant et se vide
//---------------------------------------
void AffCarreRemplir(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(177,199,1);
  }  
}
//--------------------
// Affichage ColonneTourne
//--------------------
void AffColonneTourne(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(169,176,1);
  }  
}

//--------------------
// Affichage CubeRemplir
//--------------------
void AffCubeRemplir(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(157,168,1);
  }  
}
//--------------------
// Affichage GoutteEau
//--------------------
void AffGoutteEau(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(121,156,1);
  }  
}
//---------------------
// Affichage helicopter
//---------------------
void AffHelicopter(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(111,120,1);
  }  
}

//---------------------
// Affichage en diagonal
//----------------------
void AffDiagonal(int nbfois) {
   for (int i=0; i<nbfois; i++) {
      Animation(103,110,1);
  }  
}

//------------------------
// Affichage du serpent
//------------------------
void AffSerpent(int nbfois) {
   for (int i=0; i<nbfois; i++) {
     Animation(68,102,1);
  }  
}

//------------------------
// Affichage de la spirale
//------------------------
void AffSpirale(int nbfois) {
   for (int i=0; i<nbfois; i++) {
     Animation(48,67,1);
  }  
}
//--------------------------------------
// Dessin dun carre en mouvement
// Face vers face
// Puis rotation 90 degre via diagonales
//--------------------------------------
void AffCarre(int nbfois) {
   for (int i=0; i<nbfois; i++) {
     Animation(0,5,2);  // bas -> haut
     Animation(24,29,1);  // diagonale vers face
     Animation(12,17,2); // face vers fond
     Animation(30,35,1); // face vers haut diagonale
     Animation(6,11,2); // haut vers bas
     Animation(36,41,1); // haut vers fond diagonale
     Animation(18,23,2); // Fond vers face
     Animation(42,47,1); // Fond vers bas diagonale
   }  
}
//-------------------------------
// Allume une colonne entiere
//-------------------------------
void AffColonne(int numcol, int temps) {
  if (numcol >0 && numcol<=16) {
      digitalWrite( LEDPin[numcol-1], HIGH );
      for (int i=0; i<CUBESIZE; i++) {
            digitalWrite( PlanePin[i], HIGH );
      }
      delay(temps);
      for (int i=0; i<CUBESIZE ; i++) {
            digitalWrite( PlanePin[i], LOW );
      }
      digitalWrite( LEDPin[numcol-1], LOW );
  }
}  

//-------------------------
// Allume une led au hasard
//-------------------------
void RandomLed(int nbfois) {
int randPin;
int randPlane;
  for (int i=0; i<nbfois; i++) {
      randPin = random(15); 
      randPlane = random(3);
      digitalWrite(LEDPin[randPin],HIGH);
      digitalWrite(PlanePin[randPlane],HIGH);
      delay(30);
      digitalWrite(LEDPin[randPin],LOW);
      digitalWrite(PlanePin[randPlane],LOW);
  }
}

// PROGRAMME PRINCIPAL
void loop() {
int RandAff;
     // Calcule un nombre au hasard
    // Pour lancer une fonction daffichage
    RandAff=random(1,20);
    //RandAff=20;
    switch(RandAff) {
       case 1 :
         AffCarre(2);
         break;
       case 2 :
         AffSpirale(4);
         break;
       case 3 :
         AffSerpent(4);
         break;
       case 4 :
         AffDiagonal(4);
         break;
       case 5 :
         AffHelicopter(4);
         break;
       case 6 :
         AffGoutteEau(4);
         break;
       case 7 :
         RandomLed(512);
         break;
       case 8 :
         AffCubeRemplir(4);
         break;
       case 9 :
         AffColonneTourne(4);
         break;
       case 10 :
         AffCarreRemplir(4);
         break;
       case 11 :
         AffGeyser(4);
         break;
       case 12 :
         AffSerpentHauteur(4); 
         break;
       case 13 :
         AffCarreRessort(4);
         break;
       case 14 :
         AffCubeTourne(4);
         break;
       case 15 :
         AffPetitsCubesTourne(4);
         break;
       case 16 :
         AffCarreRespire(2);
         break;
       case 17 :
         AffPlansTourne(8);
         break;
       case 18 :
         AffFleches(4);
         break;
       case 19 :
         AffCarreCroix(8);
         break; 
       case 20 :
         AffPharmacie(8);
         break; 
   }
}

